# -*- coding: utf-8 -*-
import datetime
from operator import attrgetter

from django.core.paginator import Paginator
from django.utils.translation import ugettext as _
from django.conf import settings
from django.shortcuts import redirect

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from jmb.core.utils.functions import import_class
from jmb.core.middleware.force_response import ForceResponse
from jmb.newsletter.models import Newsletter
from jmb.newsletter.cms.models import NewsletterSettings, NewsletterSearchSettings
from jmb.newsletter.cms.models import SubscriptionSettings, UnsubscriptionSettings
from jmb.newsletter2.models import Newsletter as Newsletter2


class CMSSubscriptionFormPlugin(CMSPluginBase):
    module = _('Newsletter')
    model = SubscriptionSettings
    name = _('Subscription Form')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        form_class = import_class(instance.form)
        self.render_template = instance.form_template_name

        if request.method == "POST":
            form = form_class(data=request.POST)
            if form.is_valid():
                contact = form.save()
                form.send(instance=instance, request=request)
                if instance.tracking_url:
                    raise ForceResponse(redirect(instance.tracking_url))
                self.render_template = instance.thanks_template_name
        else:
            form = form_class()
        context.update({
            'object': instance,
            'form': form,
            'placeholder': placeholder,
        })
        return context

plugin_pool.register_plugin(CMSSubscriptionFormPlugin)


class CMSUnsubscriptionFormPlugin(CMSPluginBase):
    module = _('Newsletter')
    model = UnsubscriptionSettings
    name = _('Unsubscription Form')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        form_class = import_class(instance.form)
        self.render_template = instance.form_template_name

        if request.method == "POST":
            form = form_class(request.POST)
            if form.is_valid():
                form.send(instance)
                self.render_template = instance.thanks_template_name
        else:
            form = form_class()
        context.update({
            'object': instance,
            'form': form,
            'placeholder': placeholder,
            })
        return context

plugin_pool.register_plugin(CMSUnsubscriptionFormPlugin)


class NewsletterListPlugin(CMSPluginBase):
    model = NewsletterSettings
    name = _("Newsletter List")
    module = _("Newsletter")
   
    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        ordering = instance.ordering.lower().replace(" ","").split(",")
        objects = Newsletter.objects.filter(status=settings.NEWSLETTER_STATUS_SENT)
        objects2 = Newsletter2.objects.filter(status=settings.NEWSLETTER2_STATUS_SENT)
        page = 1

        ## inclusioni
        if instance.mailinglist_slugs_included:
            mailinglist_slugs = instance.mailinglist_slugs_included.lower().replace(" ","").split(",")
            objects = objects.filter(mailing_list__slug__in = mailinglist_slugs)
        if instance.slugs_included:
            slugs = instance.slugs_included.lower().replace(" ","").split(",")
            objects = objects.filter(slug__in=slugs)
            objects2 = objects2.filter(slug__in=slugs)

        ## esclusioni
        if instance.slugs_excluded:
            excluded_slugs = instance.slugs_excluded.lower().replace(" ","").split(",")
            objects = objects.exclude(slug__in=excluded_slugs)
            objects2 = objects2.exclude(slug__in=excluded_slugs)
        if instance.mailinglist_slugs_excluded:
            excluded_mailinglist_slugs = instance.mailinglist_slugs_excluded.lower().replace(" ","").split(",")
            objects = objects.exclude(mailing_list__slug__in = excluded_mailinglist_slugs)

        ## datetime filters
        if instance.sending_date_begin:
            objects = objects.filter(sending_date__gte=instance.sending_date_begin)
            objects2 = objects2.filter(sending_date__gte=instance.sending_date_begin)
        if instance.sending_date_end:
            objects = objects.filter(sending_date__lte=instance.sending_date_end)
            objects2 = objects2.filter(sending_date__lte=instance.sending_date_end)

        request = context['request']
        par_get = {}
        query_string = None
        if request.GET:

            for key in request.GET.keys():
                if request.GET.get(key) != "":
                    par_get[key] = request.GET.get(key)

            if 'obj' in par_get and par_get['obj'] == 'newsletter':
                query_dict = {}
                query_list = []
                for term in request.META['QUERY_STRING'].split("&"):
                    term_split = term.split("=")
                    if len(term_split)>1:
                        query_dict[term_split[0]] = term_split[1]
                for key, value in query_dict.items():
                    if key != 'page' and key != 'obj_page' :
                        query_list.append(key+"="+value)
                query_string = "&".join(query_list)

            if "page" in par_get and par_get['obj_page'] == 'newsletter':
                page = par_get['page']

            # cancellazione per evitare interferenze con altre ricerche
            if "page" in par_get:
                del par_get['page']

            # cancellazione per evitare interferenze con altre ricerche
            if "obj_page" in par_get:
                del par_get['obj_page']

            if "obj" in par_get and par_get['obj'] == 'newsletter':
                par_get.pop('obj')
                objects = objects.filter(**par_get)
                objects2 = objects2.filter(**par_get)


        # FIXME: modificare nel caso non ci sia l'import di Newsletter2
        obj_sequence = list(objects) + list(objects2)

        # ordering
        for order_field_name in ordering:
            order_field_name, rev = (order_field_name[1:], True) if order_field_name[0] == '-' else (order_field_name, True)
            try:
                if Newsletter._meta.get_field(order_field_name) and Newsletter2._meta.get_field(order_field_name):
                    field_in_both = True
            except Exception, e:
                field_in_both = False
            if field_in_both:
                if order_field_name == "sending_date":
                    obj_sequence = sorted(obj_sequence, key=get_sending_date, reverse=rev)
                else:
                    obj_sequence = sorted(obj_sequence, key=attrgetter(order_field_name), reverse=rev)
        for obj in obj_sequence:
            if isinstance(obj, Newsletter):
                setattr(obj, 'nl_type', 1)
            elif isinstance(obj, Newsletter2):
                setattr(obj, 'nl_type', 2)
        paginator = Paginator(obj_sequence, instance.objects_per_page)
        objects = paginator.page(page)

        context.update({
            'objects': objects,
            'instance': instance,
            'placeholder': placeholder,
            'query_string': query_string
            })

        return context



def get_sending_date(x):
    """
    used for sorting NewsletterListPlugin: prevent TypeError for datetime object comparison with None objects
    """
    return x.sending_date or datetime.datetime(datetime.MINYEAR, 1, 1)

plugin_pool.register_plugin(NewsletterListPlugin)

class NewsletterSearchPlugin(CMSPluginBase):

    model = NewsletterSearchSettings
    name = _("Newsletter Search")
    module = _("Newsletter")

    def render(self, context, instance, placeholder):
        request = context['request']
        self.render_template = instance.template_name
        form_class = import_class(instance.form)
        try:
            if request.GET:
                form = form_class(request.GET)
            else:
                form = form_class()
        except:
            form = form_class()
        context.update({
            'form':form,
            'instance': instance,
            'placeholder': placeholder,
        })
        return context

plugin_pool.register_plugin(NewsletterSearchPlugin)
