"""Forms for newsletter"""
import datetime
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from jmb.newsletter.models import Subscriber, Newsletter
from jmb.newsletter.models import MailingList


class MailingListSubscriptionForm(forms.ModelForm):
    """Form for subscribing to a mailing list"""
    # Notes : This form will not check the uniquess of
    # the 'email' field, by defining it explictly and setting
    # it the Meta.exclude list, for allowing registration
    # to a mailing list even if the contact already exists.
    # Then the contact is always added to the subscribers field
    # of the mailing list because it will be cleaned with no
    # double.

    email = forms.EmailField(label=_('Email'), max_length=75)

    def save(self, mailing_list):
        data = self.cleaned_data
        contact, created = Subscriber.objects.get_or_create(
            email=data['email'],
        )

        mailing_list.subscribers.add(contact)
        mailing_list.unsubscribers.remove(contact)

    class Meta:
        model = Subscriber
        fields = ('email',)


class AllMailingListSubscriptionForm(MailingListSubscriptionForm):
    """Form for subscribing to all mailing list"""

    mailing_lists = forms.ModelMultipleChoiceField(
        queryset=MailingList.objects.all(),
        initial=[obj.id for obj in MailingList.objects.all()],
        label=_('Mailing lists'),
        widget=forms.CheckboxSelectMultiple())

    def save(self, mailing_list):
        data = self.cleaned_data
        contact, created = Subscriber.objects.get_or_create(
            email=data['email'],
        )

        for mailing_list in data['mailing_lists']:
            mailing_list.subscribers.add(contact)
            mailing_list.unsubscribers.remove(contact)

class MailingListUnsubscriptionForm(forms.Form):
    def __init__(self, active_subscribers, *args, **kwargs):
        super(MailingListUnsubscriptionForm, self).__init__(*args, **kwargs)
        self.fields['subscribers'].queryset = active_subscribers

    subscribers = forms.ModelMultipleChoiceField(
        queryset=Subscriber.objects.all(),
        label=_('Mailing lists'),
        widget=forms.CheckboxSelectMultiple()
    )

    def save(self):
        data = self.cleaned_data
        for active_subscriber in data['subscribers']:
            active_subscriber.date_unsubscription = datetime.datetime.now()
            active_subscriber.save()

class SearchNewsletterForm(forms.ModelForm):
    sending_date__month = forms.ChoiceField(
        label=_("Month"),
        required=False
    )
    sending_date__year = forms.ChoiceField(
        label=_("Year"),
        required=False
    )
    title__icontains = forms.CharField(
        label=_("Title ><"),
        required=False
    )
    obj = forms.CharField(
        initial="newsletter",
        widget=forms.HiddenInput
    )
    status = forms.CharField(
        initial=settings.NEWSLETTER_STATUS_SENT,
        widget=forms.HiddenInput
    )

    def __init__(self, *args, **kwargs):
        super(SearchNewsletterForm, self).__init__(*args, **kwargs)
        self.fields['sending_date__month'].choices = [('','-'*9)]+Newsletter.get_months()
        self.fields['sending_date__year'].choices = [('','-'*4)]+Newsletter.get_years()
#        self.fields['status'].choices = [(settings.NEWSLETTER_STATUS_SENT, 'sent'),]

    class Meta:
        model = Newsletter
        fields = ('status',)

