# -*- coding: utf-8 -*-
"""Models for newsletter"""
from smtplib import SMTP
from smtplib import SMTPHeloError
from datetime import datetime, time
from datetime import timedelta
from django.db import models
from django.db.models import Q
from django.utils.encoding import smart_str
from django.db import connection, transaction
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.core.validators import email_re
from django.contrib.auth.models import Group
from django.utils.encoding import force_unicode
from django.conf import settings

from jmb.mailer.models import EmailObject, MailingStatus
from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from jmb.core.db.fields import HTMLFieldTinyMCE

from jmb.newsletter.utils.tokens import tokenize

class MailingList(StatusModel, UserModel, DateModel, OrderedModel, SEOModel):
    """Mailing list"""
    name = models.CharField(
        _("Name"),
        max_length=255
    )
    slug = models.SlugField(
        _("Slug"),
        unique=True,
        max_length=100,
        default=datetime.now
    )
    description = models.TextField(
        _("Description"),
        blank=True, null=True
    )

    def get_subscribers(self):
        """ return Subscribers Contact queryset """
        return self.contacts.filter(
            date_unsubscription__isnull=True,
            status=1
        )
    subscribers = property(get_subscribers)

    def get_test_subscribers(self):
        """ return Test Subscribers Contact queryset """
        return self.subscribers.filter(
            is_test_contact=True
        )
    test_subscribers = property(get_test_subscribers)

    def get_unsubscribers(self):
        """ return Unsubscribers Contact queryset """
        return self.contacts.filter(
            date_unsubscription__isnull=False,
        )
    unsubscribers = property(get_unsubscribers)

    def get_subscriptions(self):
        """
        return number of subscribtions
        """
        return self.subscribers.count()
    subscriptions = property(get_subscriptions)

    def get_test_subscriptions(self):
        """
        return number of test subscribtions
        """
        return self.test_subscribers.count()
    test_subscriptions = property(get_test_subscriptions)

    def get_unsubscriptions(self):
        """
        return number of unsubscribtions
        """
        return self.unsubscribers.count()
    unsubscriptions = property(get_unsubscriptions)

    def has_test_contact(self):
        """ return True or False if Test Contacts are present """
        if self.test_subscribers.count() > 0:
            return True
        else:
            return False
    has_test_contact = property(has_test_contact)

    def add_subscriber(self, object):
        """
        to add subscriber to this mailinglist
        subscriber is contact object
        return a tuple with True and contact object if success
        else return a tuple with False and error message
        """
        if not email_re.match(object.email):
            return (False, _("Not valid email address"))

        try:
            contact = Subscriber.objects.get(
                mailinglist=self, email__iexact=object.email
            )
            return (False, _("Email already exists"))

        except:
            contact = Subscriber(
                mailinglist=self,
                content_object=object,
                email=object.email
            )
            contact.save()
            return (True, contact)

    def clone(self):
        """
        clone mailinglist without contacts
        """
        today = datetime.today()
        new_obj = MailingList()
        new_obj.name = '%s_%s' % (self.name, today.strftime('%d-%m-%Y'))
        new_obj.description = self.description
        new_obj.save()

        return new_obj

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('Mailing list')
        verbose_name_plural = _('Mailing lists')

class Subscriber(StatusModel, UserModel, DateModel, OrderedModel):
    """
    Subscription to a mailing list
    """
    mailinglist = models.ForeignKey(
        MailingList,
        verbose_name=_("Mailing list"),
        related_name='contacts'
    )

    # generic object with email field
    content_type = models.ForeignKey(
        ContentType,
        verbose_name=_("Content type"),
        blank=True, null=True,
        related_name='content_types'
    )
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = generic.GenericForeignKey(
        'content_type', 'object_id',
    )

    email =models.EmailField(
        _("Email"),
    )
    date_subscription = models.DateTimeField(
        _("Date subscription"),
        default=datetime.today
    )
    date_unsubscription = models.DateTimeField(
        _("Date unsubscription"),
        blank=True, null=True
    )
    is_test_contact = models.BooleanField(
        _("Test"),
    )

    class Meta:
        ordering = ('email',)
        verbose_name = _('Subscriber')
        verbose_name_plural = _('Subscribers')
        unique_together = ('mailinglist', 'email')


    def __unicode__(self):
        return "%s: %s" % (
            self.mailinglist.name,
            self.email
            )

    def save(self, *args, **kwargs):
        self.email = self.email.lower().strip()
        super(Subscriber, self).save(*args, **kwargs)

    def get_tokens(self):
        """Return the uid in base 36 of a contact, and a token"""
        return tokenize(self)

    def get_uidb36(self):
        """Return the uid in base 36 of a contact"""
        return self.get_tokens()[0]

    def get_token(self):
        """Return a token of a contact"""
        return self.get_tokens()[1]

    @classmethod
    def sottoscrittori_unici(cls):
        return cls.objects.filter(
            date_unsubscription__isnull=True
        ).values('email').distinct().count()

    @classmethod
    def disiscritti_unici(cls):
        return cls.objects.filter(
            date_unsubscription__isnull=False
        ).values('email').distinct().count()

class Newsletter(UserModel, DateModel, OrderedModel,SEOModel):
    """Newsletter to be sended to contacts"""

    @classmethod
    def get_years(cls, desc=True):
        output = []
        newsletter_qs = Newsletter.objects.filter(status=settings.NEWSLETTER_STATUS_SENT)
        for d in newsletter_qs.dates('sending_date','year', order='DESC' if desc else 'ASC'):
            output.append((str(d.year),str(d.year)))
        return output

    @classmethod
    def get_months(cls):
        return [
            ("1", _("January")),
            ("2", _("February")),
            ("3", _("March")),
            ("4", _("April")),
            ("5", _("May")),
            ("6", _("June")),
            ("7", _("July")),
            ("8", _("August")),
            ("9", _("September")),
            ("10", _("October")),
            ("11", _("November")),
            ("12", _("December"))
        ]

    status = models.IntegerField(
        _('Newsletter status'),
        choices= settings.NEWSLETTER_STATUS_CHOICES,
        default=settings.NEWSLETTER_STATUS_DRAFT
    )
    title = models.CharField(
        _("Title"),
        max_length=255,
        help_text=_(
            "You can use the '{{ UNIQUE_KEY }}' variable for unique identifier within the newsletter's title."
        )
    )
    slug = models.SlugField(
        _("Slug"),
        help_text=_('Used for displaying the newsletter on the site.'),
        unique=True
    )
    content = HTMLFieldTinyMCE(
        _('Content'),
        help_text=_('Write HTML code or paste an URL.'),
    )
    content_web = HTMLFieldTinyMCE(
        _('Content WEB'),
        help_text=_('Write web HTML code.'),
    )
    template = models.ForeignKey(
        "NewsletterTemplate",
        verbose_name=_("template"),
    )
    mailing_list = models.ForeignKey(
        MailingList,
        related_name = 'newsletters',
        verbose_name=_('Mailing list')
    )
    sending_date = models.DateTimeField(
        _('Sending date'),
        default=datetime.now
    )

    def clone(self):
        """
        clone Newsletter
        """
        today = datetime.today()
        new_obj = Newsletter()
        new_obj.title = '%s_%s' % (self.title, today.strftime('%d-%m-%Y'))
        new_obj.slug = new_obj.title
        new_obj.content = self.content
        new_obj.mailing_list = self.mailing_list
        new_obj.server = self.server
        new_obj.header_sender = self.header_sender
        new_obj.header_reply= self.header_reply
        new_obj.save()

        return new_obj

    def subscriptions(self):
        """
        return number of subscribtions
        contact is a member of mailing list of newsletter
        """
        return self.mailing_list.subscribers.count()
    subscriptions = property(subscriptions)

    def unsubscriptions(self):
        """
        return number of unsubscribtions
        contact is a member of mailing list of newsletter
        """
        return self.mailing_list.unsubscribers.count()
    unsubscriptions = property(unsubscriptions)

    def unsubscriptions_perc(self):
        """
        return percentage of unsubscribtions
        """
        if self.subscriptions != 0:
            return float(self.unsubscriptions) / float(self.subscriptions) * 100
        return float(0)
    unsubscriptions_perc = property(unsubscriptions_perc)

    def get_contact(self, email):
        """
        return an object Contact of Newsletter passing an email
        """
        contacts = self.mailing_list.contacts.filter(email=email)
        if contacts:
            return contacts[0]
        return None

    def get_email_objects(self):
        """
        return EmailObjects by GenericForeignKey
        """
        newsletter_type = ContentType.objects.get_for_model(self)
        email_objects = EmailObject.objects.filter(
            content_type__pk=newsletter_type.id,
            object_id=self.id
        )
        return email_objects

    def get_email_mailing_status(self):
        newsletter_type = ContentType.objects.get_for_model(self)
        email_mailing_status = MailingStatus.objects.filter(
            email_object__content_type__pk=newsletter_type.id,
            email_object__object_id=self.id
        )
        return email_mailing_status

    def get_default_template_path(self):
        return self.template.file.path

    def get_default_title(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return ('newsletter_newsletter_preview', (self.slug,))

    @models.permalink
    def get_historic_url(self):
        return ('newsletter_newsletter_historic', (self.slug,))

    @models.permalink
    def get_statistics_url(self):
        return ('newsletter_newsletter_statistics', (self.slug,))

    def __unicode__(self):
        return self.title

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('Newsletter')
        verbose_name_plural = _('Newsletters')
        permissions = (
            ('can_change_status', 'Can change status'),
            ('view_newsletter',   'Can view newsletter'),
            )

class NewsletterTemplate(UserModel, DateModel, OrderedModel):
    name = models.CharField(
        _("Name"),
        unique=True,
        max_length=255,
    )
    file = models.FileField(
        upload_to=settings.NEWSLETTER_TEMPLATE_PATH,
        verbose_name=_("Template"),
    )

    class Meta:
        ordering = ('name',)
        verbose_name = _('Template')
        verbose_name_plural = _('Templates')

    def __unicode__(self):
        return "%s" % self.name

class Settings (DateModel, UserModel):
    name = models.CharField(
        _("Name"),
        max_length = 100,
    )
    value = models.CharField(
        _("Value"),
        blank = True, null = True,
        max_length = 500,
    )
    description = models.TextField(
        _("Description"),
        blank = True, null = True,
    )
    type = models.CharField(
        _("Type"),
        max_length = 20,
        choices = settings.NEWSLETTER_SETTINGS_TYPE,
    )

    class Meta:
        verbose_name = _("Settings")
        verbose_name_plural = _("Settings")

    def __unicode__(self):
        return u"%s --> %s" % (self.name, self.value)

if settings.CMS_ENABLED:
    import jmb.newsletter.cms.plugins
    import jmb.newsletter.cms.app